DEVLIST = [
    719195224,  # @xditya
    1322549723,  # @danish_00
    1903729401,  # @its_buddhhu
    1303895686,  # @Sipak_OP
    611816596,  # @Arnab431
    1318486004,  # @sppidy
    803243487,  # @hellboi_atul
]

ULTROID_IMAGES = [
    f"https://telegra.ph/file/{_}.jpg"
    for _ in [
        "b8cdf9fcd5a82e94c2d06",
        "3c25230ae30d246194eba",
        "b01715a61b9e876c0d45d",
        "4ceaf720a96a24527ecff",
        "a96223b574f29f3f0d184",
        "6e081d339a01cc6190393",
    ]
]

stickers = [
    "CAADAQADeAIAAm_BZBQh8owdViocCAI",
    "CAADAQADegIAAm_BZBQ6j8GpKtnrSgI",
    "CAADAQADfAIAAm_BZBQpqC84n9JNXgI",
    "CAADAQADfgIAAm_BZBSxLmTyuHvlzgI",
    "CAADAQADgAIAAm_BZBQ3TZaueMkS-gI",
    "CAADAQADggIAAm_BZBTPcbJMorVVsQI",
    "CAADAQADhAIAAm_BZBR3lnMZRdsYxAI",
    "CAADAQADhgIAAm_BZBQGQRx4iaM4pQI",
    "CAADAQADiAIAAm_BZBRRF-cjJi_QywI",
    "CAADAQADigIAAm_BZBQQJwfzkqLM0wI",
    "CAADAQADjAIAAm_BZBQSl5GSAT0viwI",
    "CAADAQADjgIAAm_BZBQ2xU688gfHhQI",
    "CAADAQADkAIAAm_BZBRGuPNgVvkoHQI",
    "CAADAQADpgIAAm_BZBQAAZr0SJ5EKtQC",
    "CAADAQADkgIAAm_BZBTvuxuayqvjhgI",
    "CAADAQADlAIAAm_BZBSMZdWN2Yew1AI",
    "CAADAQADlQIAAm_BZBRXyadiwWGNkwI",
    "CAADAQADmAIAAm_BZBQDoB15A1jS1AI",
    "CAADAQADmgIAAm_BZBTnOLQ8_d72vgI",
    "CAADAQADmwIAAm_BZBTve1kgdG0Y5gI",
    "CAADAQADnAIAAm_BZBQUMyFiylJSqQI",
    "CAADAQADnQIAAm_BZBSMAe2V4pwhNgI",
    "CAADAQADngIAAm_BZBQ06D92QL_vywI",
    "CAADAQADnwIAAm_BZBRw7UAbr6vtEgI",
    "CAADAQADoAIAAm_BZBRkv9DnGPXh_wI",
    "CAADAQADoQIAAm_BZBQwI2NgQdyKlwI",
    "CAADAQADogIAAm_BZBRPHJF3XChVLgI",
    "CAADAQADowIAAm_BZBThpas7rZD6DAI",
    "CAADAQADpAIAAm_BZBQcC2DpZcCw1wI",
    "CAADAQADpQIAAm_BZBQKruTcEU4ntwI",
]



# Ultroid - UserBot
# Copyright (C) 2021-2022 TeamUltroid
#
# This file is a part of < https://github.com/TeamUltroid/Ultroid/ >
# PLease read the GNU Affero General Public License in
# <https://github.com/TeamUltroid/pyUltroid/blob/main/LICENSE>.

from .. import adB


def get_channels():  # Returns List
    return adB.get_key("BROADCAST") or []


def is_channel_added(id_):
    return id_ in get_channels()


def add_channel(id_):
    channels = get_channels()
    if id_ not in channels:
        channels.append(id_)
        adB.set_key("BROADCAST", channels)
    return True


def rem_channel(id_):
    channels = get_channels()
    if id_ in channels:
        channels.remove(id_)
        adB.set_key("BROADCAST", channels)
        return True
    return False



# AyiinXd
# Copyright (C) 2022-2023 @AyiinXd
#
# This file is a part of < https://github.com/AyiinXd/AyiinXd >
# PLease read the GNU Affero General Public License in
# <https://www.github.com/AyiinXd/AyiinXd/blob/main/LICENSE/>.
#
# FROM AyiinXd <https://github.com/AyiinXd/AyiinXd>
# t.me/AyiinChat & t.me/AyiinSupport

# ========================×========================
#            Jangan Hapus Credit Ngentod
# ========================×========================


import os


def where_hosted():
    if os.getenv("DYNO"):
        return "Heroku"
    if os.getenv("RAILWAY_STATIC_URL"):
        return "Railway"
    if os.getenv("KUBERNETES_PORT"):
        return "Qovery"
    if os.getenv("WINDOW") and os.getenv("WINDOW") != "0":
        return "Windows"
    if os.getenv("RUNNER_USER") or os.getenv("HOSTNAME"):
        return "Github actions"
    if os.getenv("ANDROID_ROOT"):
        return "Termux"
    return "VPS"


if where_hosted() == "VPS":
    def _ask_input():
        # Ask for Input even on Vps and other platforms.
        def new_input(*args, **kwargs):
            raise EOFError("args=" + str(args) + ", kwargs=" + str(kwargs))

        __builtins__["input"] = new_input

    _ask_input()



from .. import adB


def get_all_channels() -> dict:
    """List all chats where channels are banned."""
    return adB.get_key("AUTOBAN_CHANNELS") or {}


def is_autoban_enabled(chat_id: int) -> bool:
    """Check whether channels are banned in a specific chat or not."""
    return chat_id in get_all_channels()


def add_channel(chat_id: int) -> bool:
    """Enable channel ban in a given chat."""
    if not is_autoban_enabled(int(chat_id)):
        channels = get_all_channels()
        channels[int(chat_id)] = []
        return adB.set_key("AUTOBAN_CHANNELS", channels)


def del_channel(chat_id: int) -> bool:
    """Disable channel ban in a given chat."""
    if is_autoban_enabled(chat_id):
        channels = get_all_channels()
        channels.pop(int(chat_id))
        return adB.set_key("AUTOBAN_CHANNELS", channels)


def get_whitelisted_channels(chat_id: int) -> list:
    """Get list of whitelisted channels in a given chat."""
    return get_all_channels()[chat_id] if is_autoban_enabled(chat_id) else []


def is_whitelisted(chat_id: int, channel_id: int) -> bool:
    """Check whether given channel is whitelisted in given chat or not."""
    return channel_id in get_whitelisted_channels(chat_id)


def add_to_whitelist(chat_id: int, channel_id: int) -> bool:
    """Add a channel in whitelist in a chat."""
    if is_autoban_enabled(chat_id):
        if not is_whitelisted(chat_id, channel_id):
            channels = get_all_channels()
            channels[int(chat_id)].append(int(channel_id))
            return adB.set_key("AUTOBAN_CHANNELS", channels)


def del_from_whitelist(chat_id: int, channel_id: int) -> bool:
    """Remove a channel from whitelist in a chat."""
    if is_autoban_enabled(chat_id):
        if is_whitelisted(chat_id, channel_id):
            channels = get_all_channels()
            channels[int(chat_id)].remove(int(channel_id))
            return adB.set_key("AUTOBAN_CHANNELS", channels)



# py - Ayiin
# Copyright (C) 2022-2023 @AyiinXd
#
# This file is a part of < https://github.com/AyiinXd/pyAyiin >
# PLease read the GNU Affero General Public License in
# <https://www.github.com/AyiinXd/pyAyiin/blob/main/LICENSE/>.
#
# FROM py-Ayiin <https://github.com/AyiinXd/pyAyiin>
# t.me/AyiinChat & t.me/AyiinSupport


# ========================×========================
#            Jangan Hapus Credit Ngentod
# ========================×========================

import asyncio
import logging
import sys
import time
from aiohttp import ClientSession

from pyAyiin.Clients import *
from pyAyiin.methods import *
from pyAyiin.pyrogram import AyiinMethods
from pyAyiin.pyrogram import eod, eor
from pyAyiin.xd import GenSession
from pyAyiin.telethon.ayiin import *


# Bot Logs setup:
logging.basicConfig(
    format="[%(name)s] - [%(levelname)s] - %(message)s",
    level=logging.INFO,
)
logging.getLogger("pyAyiin").setLevel(logging.INFO)
logging.getLogger("fipper").setLevel(logging.ERROR)
logging.getLogger("fipper.client").setLevel(logging.ERROR)
logging.getLogger("fipper.session.auth").setLevel(logging.ERROR)
logging.getLogger("fipper.session.session").setLevel(logging.ERROR)


logs = logging.getLogger(__name__)


__copyright__ = "Copyright (C) 2022-present AyiinXd <https://github.com/AyiinXd>"
__license__ = "GNU General Public License v3.0 (GPL-3.0)"
__version__ = "0.1.8"
ayiin_ver = "0.0.6"


adB = AyiinDB()

DEVS = [
    607067484, # Ayiin
    # 997461844, Ayang_Ayiin
    # 1905050903, Ayiin
    2130526178, # Alfa
]

StartTime = time.time()


class PyrogramXd(AyiinMethods, GenSession, Methods):
    pass


class TelethonXd(AyiinMethod, GenSession, Methods):
    pass


suc_msg = (f"""
========================×========================
           Credit Py-Ayiin {__version__}
========================×========================
"""
)

fail_msg = (f"""
========================×========================
      Commit Yang Bener Bego Biar Gak Error
           Credit Py-Ayiin {__version__}
========================×========================
"""
)

start_bot = (f"""
========================×========================
         Starting AyiinUbot Version {ayiin_ver}
        Copyright (C) 2022-present AyiinXd
========================×========================
"""
)

run_as_module = False

if sys.argv[0] == "-m":
    run_as_module = True

    from .decorator import *

    print("\n\n" + __copyright__ + "\n" + __license__)
    print(start_bot)

    update_envs()

    CMD_HELP = {}
    adB = AyiinDB()
    aiosession = ClientSession()
    loop = asyncio.get_event_loop()
    HOSTED_ON = where_hosted()
    Yins = VcTools()
else:
    print(suc_msg)
    print("\n\n" + __copyright__ + "\n" + __license__)
    print(fail_msg)

    adB = AyiinDB()
    aiosession = ClientSession()
    loop = asyncio.get_event_loop()
    HOSTED_ON = where_hosted()



# Ultroid - UserBot
# Copyright (C) 2021-2022 TeamUltroid
#
# This file is a part of < https://github.com/TeamUltroid/Ultroid/ >
# PLease read the GNU Affero General Public License in
# <https://github.com/TeamUltroid/pyUltroid/blob/main/LICENSE>.


from .. import adB


def get_stuff():
    return adB.get_key("ASST_CMDS") or {}


def add_cmd(cmd, msg, media, button):
    ok = get_stuff()
    ok.update({cmd: {"msg": msg, "media": media, "button": button}})
    return adB.set_key("ASST_CMDS", ok)


def rem_cmd(cmd):
    ok = get_stuff()
    if ok.get(cmd):
        ok.pop(cmd)
        return adB.set_key("ASST_CMDS", ok)


def cmd_reply(cmd):
    ok = get_stuff()
    if ok.get(cmd):
        okk = ok[cmd]
        return okk["msg"], okk["media"], okk["button"] if ok.get("button") else None
    return


def list_cmds():
    ok = get_stuff()
    return ok.keys()



# Ultroid - UserBot
# Copyright (C) 2021-2022 TeamUltroid
#
# This file is a part of < https://github.com/TeamUltroid/Ultroid/ >
# PLease read the GNU Affero General Public License in
# <https://github.com/TeamUltroid/pyUltroid/blob/main/LICENSE>.

from .. import adB


def get_approved():
    return adB.get_key("PMPERMIT") or []


def approve_user(id):
    ok = get_approved()
    if id in ok:
        return True
    ok.append(id)
    return adB.set_key("PMPERMIT", ok)


def disapprove_user(id):
    ok = get_approved()
    if id in ok:
        ok.remove(id)
        return adB.set_key("PMPERMIT", ok)


def is_approved(id):
    return id in get_approved()



# Ultroid - UserBot
# Copyright (C) 2021-2022 TeamUltroid
#
# This file is a part of < https://github.com/TeamUltroid/Ultroid/ >
# PLease read the GNU Affero General Public License in
# <https://github.com/TeamUltroid/pyUltroid/blob/main/LICENSE>.

from .. import adB


def list_gbanned():
    return adB.get_key("GBAN") or {}


def gban(user, reason):
    ok = list_gbanned()
    ok.update({int(user): reason or "No Reason. "})
    return adB.set_key("GBAN", ok)


def ungban(user):
    ok = list_gbanned()
    if ok.get(int(user)):
        del ok[int(user)]
        return adB.set_key("GBAN", ok)


def is_gbanned(user):
    ok = list_gbanned()
    if ok.get(int(user)):
        return ok[int(user)]


def gmute(user):
    ok = list_gmuted()
    ok.append(int(user))
    return adB.set_key("GMUTE", ok)


def ungmute(user):
    ok = list_gmuted()
    if user in ok:
        ok.remove(int(user))
        return adB.set_key("GMUTE", ok)


def is_gmuted(user):
    return int(user) in list_gmuted()


def list_gmuted():
    return adB.get_key("GMUTE") or []



# py - Ayiin
# Copyright (C) 2022-2023 @AyiinXd
#
# This file is a part of < https://github.com/AyiinXd/pyAyiin >
# PLease read the GNU Affero General Public License in
# <https://www.github.com/AyiinXd/pyAyiin/blob/main/LICENSE/>.
#
# FROM py-Ayiin <https://github.com/AyiinXd/pyAyiin>
# t.me/AyiinChat & t.me/AyiinSupport


# ========================×========================
#            Jangan Hapus Credit Ngentod
# ========================×========================

import asyncio
import multiprocessing
import importlib
import os
import ssl
import sys
from concurrent.futures import ThreadPoolExecutor
from functools import partial, wraps

from fipper import Client
from ..pyrogram import eor, section
from ..Clients import *


try:
    import certifi
except ImportError:
    certifi = None


try:
    from git import Repo
    from git.exc import GitCommandError, InvalidGitRepositoryError, NoSuchPathError
except ImportError:
    Repo = None


try:
    import heroku3
except ImportError:
    heroku3 = None


def update_envs():
    """Update Var. attributes to adB"""
    from .. import adB

    for envs in list(os.environ):
        if envs in ["LOG_CHAT", "BOT_TOKEN"] or envs in adB.keys():
            adB.set_key(envs, os.environ[envs])


def run_async(function):
    @wraps(function)
    async def wrapper(*args, **kwargs):
        return await asyncio.get_event_loop().run_in_executor(
            ThreadPoolExecutor(max_workers=multiprocessing.cpu_count() * 5),
            partial(function, *args, **kwargs),
        )

    return wrapper

class Helpers(object):
    def import_module(
        self,
        path: str,
        exclude: list = [],
        display_module: bool = True
    ):
    
        listbin = []
        listbin.clear()
    
        if not os.path.exists(path):
            return print(f"No path found: {path}")
    
        modules = []
        modules.clear()
    
        for x in os.listdir(path):
            if x.endswith(".py"):
                if x not in ["__pycache__", "__init__.py"]:
                    modules.append(x.replace(".py", ""))
    
        py_path_raw = ".".join(path.split("/"))
        py_path = py_path_raw[0:len(py_path_raw) - 1]
    
        count = 0
        for x in modules:
            if x not in exclude:
                importlib.import_module(py_path + "." + x)
                count += 1
                listbin.append(x)
    
        if display_module:
            data = sorted(listbin)
            for x in data:
                print(x + " Loaded !")
        return count


    async def get_paste(
        self,
        data: str,
        extension: str = "txt"
    ):
        ssl_context = ssl.create_default_context(cafile=certifi.where())
        json = {"content": data, "extension": extension}
        key = await self.async_searcher(
            url="https://spaceb.in/api/v1/documents/",
            json=json,
            ssl=ssl_context,
            post=True,
            re_json=True,
        )
        try:
            return True, key["payload"]["id"]
        except KeyError:
            if "the length must be between 2 and 400000." in key["error"]:
                return await self.get_paste(data[-400000:], extension=extension)
            return False, key["error"]
        except Exception as e:
            print(e)
            return None, str(e)

    async def install_requirements(self):
        try:
            process = await asyncio.create_subprocess_shell(
                " ".join([sys.executable, "-m", "pip", "install", '-U', "-r", "requirements.txt"]),
                stdout=asyncio.subprocess.PIPE,
                stderr=asyncio.subprocess.PIPE,
            )
            await process.communicate()
            return process.returncode
        except Exception as e:
            return repr(e)
    
    @run_async
    def gen_chlog(self, repo, diff):
        """Generate Changelogs..."""
        from pyAyiin import ayiin_ver
        
        UPSTREAM_REPO_URL = (
            Repo().remotes[0].config_reader.get("url").replace(".git", "")
        )
        ac_br = repo.active_branch.name
        ch_log = tldr_log = ""
        ch = f"<b>Ayiin Ubot {ayiin_ver} updates for <a href={UPSTREAM_REPO_URL}/tree/{ac_br}>[{ac_br}]</a>:</b>"
        ch_tl = f"Ayiin Ubot {ayiin_ver} updates for {ac_br}:"
        d_form = "%d/%m/%y || %H:%M"
        for c in repo.iter_commits(diff):
            ch_log += f"\n\n# <b>{c.count()}</b> 🗓 <b>[{c.committed_datetime.strftime(d_form)}]</b>\n<b><a href={UPSTREAM_REPO_URL.rstrip('/')}/commit/{c}>[{c.summary}]</a></b> 👨‍💻 <code>{c.author}</code>"
            tldr_log += f"\n\n# {c.count()} 🗓 [{c.committed_datetime.strftime(d_form)}]\n[{c.summary}] 👨‍💻 {c.author}"
        if ch_log:
            return str(ch + ch_log), str(ch_tl + tldr_log)
        return ch_log, tldr_log
    
    async def updater(self):
        from .. import logs

        try:
            off_repo = Repo().remotes[0].config_reader.get("url").replace(".git", "")
        except Exception as er:
            logs.exception(er)
            return
        try:
            repo = Repo()
        except NoSuchPathError as error:
            logs.info(f"`directory {error} is not found`")
            Repo().__del__()
            return
        except GitCommandError as error:
            logs.info(f"`Early failure! {error}`")
            Repo().__del__()
            return
        except InvalidGitRepositoryError:
            repo = Repo.init()
            origin = repo.create_remote("upstream", off_repo)
            origin.fetch()
            repo.create_head("main", origin.refs.main)
            repo.heads.main.set_tracking_branch(origin.refs.main)
            repo.heads.main.checkout(True)
        ac_br = repo.active_branch.name
        try:
            repo.create_remote("upstream", off_repo)
        except Exception as er:
            logs.info(er)
        ups_rem = repo.remote("upstream")
        ups_rem.fetch(ac_br)
        changelog, tl_chnglog = await self.gen_chlog(repo, f"HEAD..upstream/{ac_br}")
        return bool(changelog)
    
    async def restart(xd=None):
        from pyAyiin import logs
        
        if Var.HEROKU_APP_NAME and Var.HEROKU_API:
            try:
                Heroku = heroku3.from_key(Var.HEROKU_API)
                app = Heroku.apps()[Var.HEROKU_APP_NAME]
                if xd:
                    await xd.edit("`Restarting your app, please wait for a minute!`")
                app.restart()
            except BaseException as er:
                if xd:
                    return await eor(
                        xd,
                        "`HEROKU_API` or `HEROKU_APP_NAME` is wrong! Kindly re-check in config vars.",
                    )
                logs.exception(er)
        else:
            if len(sys.argv) == 1:
                os.execl(sys.executable, sys.executable, "-m", "pyAyiin")
            else:
                os.execl(
                    sys.executable,
                    sys.executable,
                    "-m",
                    "pyAyiin",
                    sys.argv[1],
                    sys.argv[2],
                    sys.argv[3],
                    sys.argv[4],
                    sys.argv[5],
                    sys.argv[6],
                )
    
    async def get_user_info(self, client: Client, user, already=False):
        if not already:
            user = await client.get_users(user)
        if not user.first_name:
            return ["Deleted account", None]
        user_id = user.id
        username = user.username
        first_name = user.first_name
        mention = user.mention
        dc_id = user.dc_id
        photo_id = user.photo.big_file_id if user.photo else None
        body = {
            "ID": user_id,
            "DC": dc_id,
            "Name": [first_name],
            "Username": [("@" + username) if username else ""],
            "Mention": [mention],
        }
        caption = section("User info", body)
        return [caption, photo_id]


    async def get_chat_info(self, client: Client, chat, already=False):
        if not already:
            chat = await client.get_chat(chat)
        chat_id = chat.id
        username = chat.username
        title = chat.title
        type_ = chat.type
        is_scam = chat.is_scam
        description = chat.description
        members = chat.members_count
        is_restricted = chat.is_restricted
        link = f"[Link](t.me/{username})" if username else "??"
        dc_id = chat.dc_id
        photo_id = chat.photo.big_file_id if chat.photo else None
        body = {
            "ID": chat_id,
            "DC": dc_id,
            "Type": type_,
            "Name": [title],
            "Username": [("@" + username) if username else ""],
            "Mention": [link],
            "Members": members,
            "Scam": is_scam,
            "Restricted": is_restricted,
            "Description": [description],
        }
        caption = section("Chat info", body)
        return [caption, photo_id]



import random

from .. import *
#from ..Clients import 


assistantdict = {}


async def get_client(assistant: int):
    if int(assistant) == 1:
        return AYIIN1
    elif int(assistant) == 2:
        return AYIIN2
    elif int(assistant) == 3:
        return AYIIN3
    elif int(assistant) == 4:
        return AYIIN4
    elif int(assistant) == 5:
        return AYIIN5


async def set_assistant(chat_id):    
    ran_assistant = random.choice(clients)
    assistantdict[chat_id] = ran_assistant
    if not assistantdict.get("CLIENT"):
        try:
            assistantdict[chat_id].update({'assistant':  ran_assistant})
        except Exception:
            assistantdict.update({chat_id: {'assistant': ran_assistant}})
    '''
    await db.update_one(
        {"chat_id": chat_id},
        {"$set": {"assistant": ran_assistant}},
        upsert=True,
    )
    '''
    userbot = await get_client(ran_assistant)
    return userbot


async def get_assistant(chat_id: int) -> str:
    assistant = assistantdict.get(chat_id)
    if assistant:
        if assistant in clients:
            userbot = await get_client(assistant)
            return userbot
        else:
            userbot = await set_assistant(chat_id)
            return userbot


async def set_calls_assistant(chat_id):
    ran_assistant = random.choice(clients)
    assistantdict[chat_id] = ran_assistant
    if not assistantdict.get("CLIENT"):
        try:
            assistantdict[chat_id].update({'assistant':  ran_assistant})
        except Exception:
            assistantdict.update({chat_id: {'assistant': ran_assistant}})
    '''
    await db.update_one(
        {"chat_id": chat_id},
        {"$set": {"assistant": ran_assistant}},
        upsert=True,
    )
    '''
    return ran_assistant


async def group_assistant(chat_id: int) -> int:
    assistant = assistantdict.get(chat_id)
    if assistant:
        if assistant in clients:
            assis = assistant
        else:
            assis = await set_calls_assistant(chat_id)
    if int(assis) == 1:
        return AYIIN1
    elif int(assis) == 2:
        return AYIIN2
    elif int(assis) == 3:
        return AYIIN3
    elif int(assis) == 4:
        return AYIIN4
    elif int(assis) == 5:
        return AYIIN5



# Ultroid - UserBot
# Copyright (C) 2021-2022 TeamUltroid
#
# This file is a part of < https://github.com/TeamUltroid/Ultroid/ >
# PLease read the GNU Affero General Public License in
# <https://github.com/TeamUltroid/pyUltroid/blob/main/LICENSE>.

from .. import adB


def get_stuff():
    return adB.get_key("NOTE") or {}


def add_note(chat, word, msg, media, button):
    ok = get_stuff()
    if ok.get(int(chat)):
        ok[int(chat)].update({word: {"msg": msg, "media": media, "button": button}})
    else:
        ok.update({int(chat): {word: {"msg": msg, "media": media, "button": button}}})
    adB.set_key("NOTE", ok)


def rem_note(chat, word):
    ok = get_stuff()
    if ok.get(int(chat)) and ok[int(chat)].get(word):
        ok[int(chat)].pop(word)
        return adB.set_key("NOTE", ok)


def rem_all_note(chat):
    ok = get_stuff()
    if ok.get(int(chat)):
        ok.pop(int(chat))
        return adB.set_key("NOTE", ok)


def get_notes(chat, word):
    ok = get_stuff()
    if ok.get(int(chat)) and ok[int(chat)].get(word):
        return ok[int(chat)][word]


def list_note(chat):
    ok = get_stuff()
    if ok.get(int(chat)):
        return "".join(f"👉 #{z}\n" for z in ok[chat])



# py - Ayiin
# Copyright (C) 2022-2023 @AyiinXd
#
# This file is a part of < https://github.com/AyiinXd/pyAyiin >
# PLease read the GNU Affero General Public License in
# <https://www.github.com/AyiinXd/pyAyiin/blob/main/LICENSE/>.
#
# FROM py-Ayiin <https://github.com/AyiinXd/pyAyiin>
# t.me/AyiinChat & t.me/AyiinSupport


# ========================×========================
#            Jangan Hapus Credit Ngentod
# ========================×========================

from fipper.types import CallbackQuery, Message


class FuncBot(object):
    async def approve_pmpermit(
        self,
        cb,
        user_ids,
        OLD_MSG,
    ):
        from ..dB.pmpermit_db import approve_user, is_approved

        
        if isinstance(cb, CallbackQuery):
            if is_approved(user_ids):
                await cb.answer("Pengguna Ini Sudah Ada Di Database.", show_alert=True)
                return
            approve_user(user_ids)
            await cb.edit_message_text("Pesan Anda Diterima Tod")
            if str(user_ids) in OLD_MSG:
                await OLD_MSG[str(user_ids)].delete()
        elif isinstance(cb, Message):
            if is_approved(user_ids):
                await cb.edit("Pengguna Ini Sudah Ada Di Database.", show_alert=True)
                return
            approve_user(user_ids)
            await cb.edit("Pesan Anda Diterima Tod")
            if str(user_ids) in OLD_MSG:
                await OLD_MSG[str(user_ids)].delete()
        
    async def disapprove_pmpermit(
        self,
        cb,
        user_ids,
    ):
        from ..dB.pmpermit_db import disapprove_user, is_approved
        
        if isinstance(cb, CallbackQuery):
            if not is_approved(user_ids):
                return await cb.answer("Pengguna Ini Tidak Ada Di Database")
            disapprove_user(user_ids)
            await cb.edit_message_text("Pesan Anda Ditolak Tod")
        elif isinstance(cb, Message):
            if not is_approved(user_ids):
                return await cb.edit("Pengguna Ini Tidak Ada Di Database")
            disapprove_user(user_ids)
            await cb.edit("Pesan Anda Ditolak Tod")



import math

from fipper.types import (
    InlineKeyboardButton
)


class ToolBot(object):
    def HelpXd(
        self,
        page_number,
        allmodules,
        prefix
    ):
        rows = 4
        column = 2
        help_modules = []
        for mod in allmodules:
            if not mod.startswith("_"):
                help_modules.append(mod)
        help_modules = sorted(help_modules)
        modules = [
            InlineKeyboardButton(
                text="{}".format(
                    x.replace("_", " ").title(),
                ),
                callback_data="pluginlist-{}|{}".format(x, page_number),
            )
            for x in help_modules
        ]
        twins = list(zip(modules[::column], modules[1::column]))
        if len(modules) % column == 1:
            twins.append((modules[-1],))
        num_pages = math.ceil(len(twins) / rows)
        mod_page = page_number % num_pages
        if len(twins) > rows:
            twins = twins[
                mod_page * rows: rows * (mod_page + 1)
            ] + [
                (
                    InlineKeyboardButton(
                        text="❮❮",
                        callback_data="{}-prev({})".format(
                            prefix, mod_page
                        ),
                    ),
                    InlineKeyboardButton(
                        text="❯❯",
                        callback_data="{}-next({})".format(
                            prefix, mod_page
                        ),
                    ),
                )
            ]
        return twins
    
    
    async def PluginXd(
        self,
        cmd_help,
        modules: str
    ):
        try:
            module_data = []
            module_data.clear()
    
            for x, y in zip(
                cmd_help.get(modules)[1].keys(),
                cmd_help.get(modules)[1].values()
            ):
                module_data.append(
                    f"<b>CMD:</b> <code>{x}</code>\n<b>INFO:</b> <code>{y}</code>\n\n"
                )
            return module_data
        except Exception:
            return None



# Ultroid - UserBot
# Copyright (C) 2021-2022 TeamUltroid
#
# This file is a part of < https://github.com/TeamUltroid/Ultroid/ >
# PLease read the GNU Affero General Public License in
# <https://github.com/TeamUltroid/pyUltroid/blob/main/LICENSE>.

from .. import adB


def get_stuff():
    return adB.get_key("ECHO") or {}


def add_echo(chat, user):
    x = get_stuff()
    if k := x.get(int(chat)):
        if user not in k:
            k.append(int(user))
        x.update({int(chat): k})
    else:
        x.update({int(chat): [int(user)]})
    return adB.set_key("ECHO", x)


def rem_echo(chat, user):
    x = get_stuff()
    if k := x.get(int(chat)):
        if user in k:
            k.remove(int(user))
        x.update({int(chat): k})
    return adB.set_key("ECHO", x)


def check_echo(chat, user):
    x = get_stuff()
    if (k := x.get(int(chat))) and int(user) in k:
        return True


def list_echo(chat):
    x = get_stuff()
    return x.get(int(chat))



# py - Ayiin
# Copyright (C) 2022-2023 @AyiinXd
#
# This file is a part of < https://github.com/AyiinXd/pyAyiin >
# PLease read the GNU Affero General Public License in
# <https://www.github.com/AyiinXd/pyAyiin/blob/main/LICENSE/>.
#
# FROM py-Ayiin <https://github.com/AyiinXd/pyAyiin>
# t.me/AyiinChat & t.me/AyiinSupport


# ========================×========================
#            Jangan Hapus Credit Ngentod
# ========================×========================

"""
Exceptions which can be raised by py-Ayiin Itself.
"""


class pyAyiinError(Exception):
    ...


class PyrogramMissingError(ImportError):
    ...


class TelethonMissingError(ImportError):
    ...


class DependencyMissingError(ImportError):
    ...


class RunningAsFunctionLibError(pyAyiinError):
    ...


class SpamFailed(Exception):
    ...


class DownloadFailed(Exception):
    ...


class DelAllFailed(Exception):
    ...


class FFmpegReturnCodeError(Exception):
    ...



from .. import adB


def add_black_chat(chat_id):
    chat = adB.get_key("BLACKLIST_CHATS") or []
    if chat_id not in chat:
        chat.append(chat_id)
        return adB.set_key("BLACKLIST_CHATS", chat)


def rem_black_chat(chat_id):
    chat = adB.get_key("BLACKLIST_CHATS") or []
    if chat_id in chat:
        chat.remove(chat_id)
        return adB.set_key("BLACKLIST_CHATS", chat)



#
# Copyright (C) 2021-2022 by TeamYukki@Github, < https://github.com/TeamYukki >.
#
# This file is part of < https://github.com/TeamYukki/YukkiMusicBot > project,
# and is released under the "GNU v3.0 License Agreement".
# Please see < https://github.com/TeamYukki/YukkiMusicBot/blob/master/LICENSE >
#
# All rights reserved.

import aiohttp
import requests

BASE = "https://batbin.me/"


async def post(url: str, *args, **kwargs):
    async with aiohttp.ClientSession() as session:
        async with session.post(url, *args, **kwargs) as resp:
            try:
                data = await resp.json()
            except Exception:
                data = await resp.text()
        return data


async def PasteBin(text):
    resp = await post(f"{BASE}api/v2/paste", data=text)
    if not resp["success"]:
        return
    link = BASE + resp["message"]
    return link


async def paste(content: str):
    resp = await post(f"{BASE}api/v2/paste", data=content)
    if not resp["success"]:
        return
    return BASE + resp["message"]


async def s_paste(message, extension="txt"):
    siteurl = "https://spaceb.in/api/v1/documents/"
    try:
        response = requests.post(
            siteurl, data={"content": message, "extension": extension}
        )
    except Exception as e:
        return {"error": str(e)}
    if response.ok:
        response = response.json()
        if response["error"] != "" and response["status"] < 400:
            return {"error": response["error"]}
        return {
            "url": f"https://spaceb.in/{response['payload']['id']}",
            "raw": f"{siteurl}{response['payload']['id']}/raw",
            "bin": "Spacebin",
        }
    return {"error": "Unable to reach spacebin."}



# Ayiin - Userbot
# Copyright (C) 2022-2023 @AyiinXd
#
# This file is a part of < https://github.com/AyiinXd/Ayiin-Userbot >
# PLease read the GNU Affero General Public License in
# <https://www.github.com/AyiinXd/Ayiin-Userbot/blob/main/LICENSE/>.
#
# FROM Ayiin-Userbot <https://github.com/AyiinXd/Ayiin-Userbot>
# t.me/AyiinChat & t.me/AyiinSupport


# ========================×========================
#            Jangan Hapus Credit Ngentod
# ========================×========================

import logging
import sys

from pyAyiin.config import Var as Variable

from ..methods._database import AyiinDB
from ..methods.helpers import Helpers
from ..methods.hosting import where_hosted

from .client import *


adB = AyiinDB()
logs = logging.getLogger(__name__)
HOSTED_ON = where_hosted()
Var = Variable()
Xd = Helpers()


async def ayiin_client(client):
    try:
        await client.join_chat("AyiinChat")
        await client.join_chat("AyiinSupport")
        await client.join_chat("StoryAyiin")
    except Exception:
        pass


clients = []
client_id = []


async def StartPyrogram():
    try:
        bot_plugins = Xd.import_module(
            "assistant/",
            display_module=False,
            exclude=Var.NO_LOAD,
        )
        logs.info(f"{bot_plugins} Total Plugins Bot")
        plugins = Xd.import_module(
            "AyiinXd/",
            display_module=False,
            exclude=Var.NO_LOAD,
        )
        logs.info(f"{plugins} Total Plugins User")
    except BaseException as e:
        logs.info(e)
        sys.exit()
    if tgbot:
        await tgbot.start()
        me = await tgbot.get_me()
        tgbot.id = me.id
        tgbot.mention = me.mention
        tgbot.username = me.username
        if me.last_name:
            tgbot.name = me.first_name + " " + me.last_name
        else:
            tgbot.name = me.first_name
        logs.info(
            f"TgBot in {tgbot.name} | [ {tgbot.id} ]"
        )
        client_id.append(tgbot.id)
    if AYIIN1:
        try:
            await AYIIN1.start()
            clients.append(1)
            await ayiin_client(AYIIN1)
            me = await AYIIN1.get_me()
            AYIIN1.id = me.id
            AYIIN1.mention = me.mention
            AYIIN1.username = me.username
            if me.last_name:
                AYIIN1.name = me.first_name + " " + me.last_name
            else:
                AYIIN1.name = me.first_name
            #AYIIN1.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN1 in {AYIIN1.name} | [ {AYIIN1.id} ]"
            )
            client_id.append(AYIIN1.id)
        except Exception as e:
            logs.info(f"[STRING_1] ERROR: {e}")
    if AYIIN2:
        try:
            await AYIIN2.start()
            clients.append(2)
            await ayiin_client(AYIIN2)
            me = await AYIIN2.get_me()
            AYIIN2.id = me.id
            AYIIN2.mention = me.mention
            AYIIN2.username = me.username
            if me.last_name:
                AYIIN2.name = me.first_name + " " + me.last_name
            else:
                AYIIN2.name = me.first_name
            #AYIIN2.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN2 in {AYIIN2.name} | [ {AYIIN2.id} ]"
            )
            client_id.append(AYIIN2.id)
        except Exception as e:
            logs.info(f"[STRING_2] ERROR: {e}")
    if AYIIN3:
        try:
            await AYIIN3.start()
            clients.append(3)
            await ayiin_client(AYIIN3)
            me = await AYIIN3.get_me()
            AYIIN3.id = me.id
            AYIIN3.mention = me.mention
            AYIIN3.username = me.username
            if me.last_name:
                AYIIN3.name = me.first_name + " " + me.last_name
            else:
                AYIIN3.name = me.first_name
            #AYIIN3.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN3 in {AYIIN3.name} | [ {AYIIN3.id} ]"
            )
            client_id.append(AYIIN3.id)
        except Exception as e:
            logs.info(f"[STRING_3] ERROR: {e}")
    if AYIIN4:
        try:
            await AYIIN4.start()
            clients.append(4)
            await ayiin_client(AYIIN4)
            me = await AYIIN4.get_me()
            AYIIN4.id = me.id
            AYIIN4.mention = me.mention
            AYIIN4.username = me.username
            if me.last_name:
                AYIIN4.name = me.first_name + " " + me.last_name
            else:
                AYIIN4.name = me.first_name
            #AYIIN4.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN4 in {AYIIN4.name} | [ {AYIIN4.id} ]"
            )
            client_id.append(AYIIN4.id)
        except Exception as e:
            logs.info(f"[STRING_4] ERROR: {e}")
    if AYIIN5:
        try:
            await AYIIN5.start()
            clients.append(5)
            await ayiin_client(AYIIN5)
            me = await AYIIN5.get_me()
            AYIIN5.id = me.id
            AYIIN5.mention = me.mention
            AYIIN5.username = me.username
            if me.last_name:
                AYIIN5.name = me.first_name + " " + me.last_name
            else:
                AYIIN5.name = me.first_name
            #AYIIN5.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN5 in {AYIIN5.name} | [ {AYIIN5.id} ]"
            )
            client_id.append(AYIIN5.id)
        except Exception as e:
            logs.info(f"[STRING_5] ERROR: {e}")
    if AYIIN6:
        try:
            await AYIIN6.start()
            clients.append(6)
            await ayiin_client(AYIIN6)
            me = await AYIIN6.get_me()
            AYIIN6.id = me.id
            AYIIN6.mention = me.mention
            AYIIN6.username = me.username
            if me.last_name:
                AYIIN6.name = me.first_name + " " + me.last_name
            else:
                AYIIN6.name = me.first_name
            #AYIIN1.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN6 in {AYIIN6.name} | [ {AYIIN6.id} ]"
            )
            client_id.append(AYIIN6.id)
        except Exception as e:
            logs.info(f"[STRING_6] ERROR: {e}")
    if AYIIN7:
        try:
            await AYIIN7.start()
            clients.append(7)
            await ayiin_client(AYIIN7)
            me = await AYIIN7.get_me()
            AYIIN7.id = me.id
            AYIIN7.mention = me.mention
            AYIIN7.username = me.username
            if me.last_name:
                AYIIN7.name = me.first_name + " " + me.last_name
            else:
                AYIIN7.name = me.first_name
            #AYIIN7.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN7 in {AYIIN7.name} | [ {AYIIN7.id} ]"
            )
            client_id.append(AYIIN7.id)
        except Exception as e:
            logs.info(f"[STRING_7] ERROR: {e}")
    if AYIIN8:
        try:
            await AYIIN8.start()
            clients.append(8)
            await ayiin_client(AYIIN8)
            me = await AYIIN8.get_me()
            AYIIN8.id = me.id
            AYIIN8.mention = me.mention
            AYIIN8.username = me.username
            if me.last_name:
                AYIIN8.name = me.first_name + " " + me.last_name
            else:
                AYIIN8.name = me.first_name
            #AYIIN8.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN8 in {AYIIN8.name} | [ {AYIIN8.id} ]"
            )
            client_id.append(AYIIN8.id)
        except Exception as e:
            logs.info(f"[STRING_8] ERROR: {e}")
    if AYIIN9:
        try:
            await AYIIN9.start()
            clients.append(9)
            await ayiin_client(AYIIN9)
            me = await AYIIN9.get_me()
            AYIIN9.id = me.id
            AYIIN9.mention = me.mention
            AYIIN9.username = me.username
            if me.last_name:
                AYIIN9.name = me.first_name + " " + me.last_name
            else:
                AYIIN9.name = me.first_name
            #AYIIN9.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN9 in {AYIIN9.name} | [ {AYIIN9.id} ]"
            )
            client_id.append(AYIIN9.id)
        except Exception as e:
            logs.info(f"[STRING_9] ERROR: {e}")
    if AYIIN10:
        try:
            await AYIIN10.start()
            clients.append(10)
            await ayiin_client(AYIIN10)
            me = await AYIIN10.get_me()
            AYIIN10.id = me.id
            AYIIN10.mention = me.mention
            AYIIN10.username = me.username
            if me.last_name:
                AYIIN10.name = me.first_name + " " + me.last_name
            else:
                AYIIN10.name = me.first_name
            #AYIIN10.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN10 in {AYIIN10.name} | [ {AYIIN10.id} ]"
            )
            client_id.append(AYIIN10.id)
        except Exception as e:
            logs.info(f"[STRING_10] ERROR: {e}")
    if AYIIN11:
        try:
            await AYIIN11.start()
            clients.append(11)
            AYIIN11.me = await AYIIN11.get_me()
            #AYIIN11.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN11 in {AYIIN11.me.first_name} | [ {AYIIN11.me.id} ]"
            )
        except Exception:
            pass
    if AYIIN12:
        try:
            await AYIIN12.start()
            clients.append(12)
            AYIIN12.me = await AYIIN12.get_me()
            #AYIIN12.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN12 in {AYIIN12.me.first_name} | [ {AYIIN12.me.id} ]"
            )
        except Exception:
            pass
    if AYIIN13:
        try:
            await AYIIN13.start()
            clients.append(13)
            AYIIN13.me = await AYIIN13.get_me()
            #AYIIN13.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN13 in {AYIIN13.me.first_name} | [ {AYIIN13.me.id} ]"
            )
        except Exception:
            pass
    if AYIIN14:
        try:
            await AYIIN14.start()
            clients.append(14)
            AYIIN14.me = await AYIIN14.get_me()
            #AYIIN14.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN14 in {AYIIN14.me.first_name} | [ {AYIIN14.me.id} ]"
            )
        except Exception:
            pass
    if AYIIN15:
        try:
            await AYIIN15.start()
            clients.append(15)
            AYIIN15.me = await AYIIN15.get_me()
            #AYIIN15.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN15 in {AYIIN15.me.first_name} | [ {AYIIN15.me.id} ]"
            )
        except Exception:
            pass
    if AYIIN16:
        try:
            await AYIIN16.start()
            clients.append(16)
            AYIIN16.me = await AYIIN16.get_me()
           # AYIIN16.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN16 in {AYIIN16.me.first_name} | [ {AYIIN16.me.id} ]"
            )
        except Exception:
            pass
    if AYIIN17:
        try:
            await AYIIN17.start()
            clients.append(17)
            AYIIN17.me = await AYIIN17.get_me()
            #AYIIN17.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN17 in {AYIIN17.me.first_name} | [ {AYIIN17.me.id} ]"
            )
        except Exception:
            pass
    if AYIIN18:
        try:
            await AYIIN18.start()
            AYIIN18.me = await AYIIN18.get_me()
            clients.append(18)
            #AYIIN18.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN18 in {AYIIN18.me.first_name} | [ {AYIIN18.me.id} ]"
            )
        except Exception:
            pass
    if AYIIN19:
        try:
            await AYIIN19.start()
            clients.append(19)
            AYIIN19.me = await AYIIN19.get_me()
            #AYIIN19.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN19 in {AYIIN19.me.first_name} | [ {AYIIN19.me.id} ]"
            )
        except Exception:
            pass
    if AYIIN20:
        try:
            await AYIIN20.start()
            clients.append(20)
            AYIIN20.me = await AYIIN20.get_me()
            #AYIIN20.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN20 in {AYIIN20.me.first_name} | [ {AYIIN20.me.id} ]"
            )
        except Exception:
            pass
    if AYIIN21:
        try:
            await AYIIN21.start()
            clients.append(21)
            AYIIN21.me = await AYIIN21.get_me()
            #AYIIN21.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN21 in {AYIIN21.me.first_name} | [ {AYIIN21.me.id} ]"
            )
        except Exception:
            pass
    if AYIIN22:
        try:
            await AYIIN22.start()
            clients.append(22)
            AYIIN22.me = await AYIIN22.get_me()
            #AYIIN22.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN22 in {AYIIN22.me.first_name} | [ {AYIIN22.me.id} ]"
            )
        except Exception:
            pass
    if AYIIN23:
        try:
            await AYIIN23.start()
            clients.append(23)
            AYIIN23.me = await AYIIN23.get_me()
            #AYIIN23.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN23 in {AYIIN23.me.first_name} | [ {AYIIN23.me.id} ]"
            )
        except Exception:
            pass
    if AYIIN24:
        try:
            await AYIIN24.start()
            clients.append(24)
            AYIIN24.me = await AYIIN24.get_me()
            #AYIIN24.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN24 in {AYIIN24.me.first_name} | [ {AYIIN24.me.id} ]"
            )
        except Exception:
            pass
    if AYIIN25:
        try:
            await AYIIN25.start()
            clients.append(25)
            AYIIN25.me = await AYIIN25.get_me()
            #AYIIN25.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN25 in {AYIIN25.me.first_name} | [ {AYIIN25.me.id} ]"
            )
        except Exception:
            pass
    if AYIIN26:
        try:
            await AYIIN26.start()
            clients.append(26)
            AYIIN26.me = await AYIIN26.get_me()
            #AYIIN26.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN26 in {AYIIN26.me.first_name} | [ {AYIIN26.me.id} ]"
            )
        except Exception:
            pass
    if AYIIN27:
        try:
            await AYIIN27.start()
            clients.append(27)
            AYIIN27.me = await AYIIN27.get_me()
            #AYIIN27.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN27 in {AYIIN27.me.first_name} | [ {AYIIN27.me.id} ]"
            )
        except Exception:
            pass
    if AYIIN28:
        try:
            await AYIIN28.start()
            clients.append(28)
            AYIIN28.me = await AYIIN28.get_me()
            #AYIIN28.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN28 in {AYIIN28.me.first_name} | [ {AYIIN28.me.id} ]"
            )
        except Exception:
            pass
    if AYIIN29:
        try:
            await AYIIN29.start()
            AYIIN29.me = await AYIIN29.get_me()
            clients.append(29)
            #AYIIN29.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN29 in {AYIIN29.me.first_name} | [ {AYIIN29.me.id} ]"
            )
        except Exception:
            pass
    if AYIIN30:
        try:
            await AYIIN30.start()
            clients.append(30)
            AYIIN30.me = await AYIIN30.get_me()
            #AYIIN30.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN30 in {AYIIN30.me.first_name} | [ {AYIIN30.me.id} ]"
            )
        except Exception:
            pass
    if AYIIN31:
        try:
            await AYIIN31.start()
            clients.append(31)
            AYIIN31.me = await AYIIN31.get_me()
            #AYIIN31.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN31 in {AYIIN31.me.first_name} | [ {AYIIN31.me.id} ]"
            )
        except Exception:
            pass
    if AYIIN32:
        try:
            await AYIIN32.start()
            clients.append(32)
            AYIIN32.me = await AYIIN32.get_me()
            #AYIIN32.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN32 in {AYIIN32.me.first_name} | [ {AYIIN32.me.id} ]"
            )
        except Exception:
            pass
    if AYIIN33:
        try:
            await AYIIN33.start()
            clients.append(33)
            AYIIN33.me = await AYIIN33.get_me()
            #AYIIN33.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN33 in {AYIIN33.me.first_name} | [ {AYIIN33.me.id} ]"
            )
        except Exception:
            pass
    if AYIIN34:
        try:
            await AYIIN34.start()
            clients.append(34)
            AYIIN34.me = await AYIIN34.get_me()
            #AYIIN34.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN34 in {AYIIN34.me.first_name} | [ {AYIIN34.me.id} ]"
            )
        except Exception:
            pass
    if AYIIN35:
        try:
            await AYIIN35.start()
            clients.append(35)
            AYIIN35.me = await AYIIN35.get_me()
            #AYIIN35.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN35 in {AYIIN35.me.first_name} | [ {AYIIN35.me.id} ]"
            )
        except Exception:
            pass
    if AYIIN36:
        try:
            await AYIIN36.start()
            clients.append(36)
            AYIIN36.me = await AYIIN36.get_me()
            #AYIIN36.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN36 in {AYIIN36.me.first_name} | [ {AYIIN36.me.id} ]"
            )
        except Exception:
            pass
    if AYIIN37:
        try:
            await AYIIN37.start()
            clients.append(37)
            AYIIN37.me = await AYIIN37.get_me()
            #AYIIN37.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN37 in {AYIIN37.me.first_name} | [ {AYIIN37.me.id} ]"
            )
        except Exception:
            pass
    if AYIIN38:
        try:
            await AYIIN38.start()
            clients.append(38)
            AYIIN38.me = await AYIIN38.get_me()
            #AYIIN38.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN38 in {AYIIN38.me.first_name} | [ {AYIIN38.me.id} ]"
            )
        except Exception:
            pass
    if AYIIN39:
        try:
            await AYIIN39.start()
            clients.append(39)
            AYIIN39.me = await AYIIN39.get_me()
            #AYIIN39.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN39 in {AYIIN39.me.first_name} | [ {AYIIN39.me.id} ]"
            )
            client_id.append(AYIIN39.me.id)
        except Exception:
            pass
    if AYIIN40:
        try:
            await AYIIN40.start()
            clients.append(40)
            AYIIN40.me = await AYIIN40.get_me()
            #AYIIN40.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN40 in {AYIIN40.me.first_name} | [ {AYIIN40.me.id} ]"
            )
            client_id.append(AYIIN40.me.id)
        except Exception:
            pass
    if AYIIN41:
        try:
            await AYIIN41.start()
            clients.append(41)
            AYIIN41.me = await AYIIN41.get_me()
            #AYIIN41.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN41 in {AYIIN41.me.first_name} | [ {AYIIN41.me.id} ]"
            )
            client_id.append(AYIIN41.me.id)
        except Exception:
            pass
    if AYIIN42:
        try:
            await AYIIN42.start()
            clients.append(42)
            AYIIN42.me = await AYIIN42.get_me()
            #AYIIN42.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN42 in {AYIIN42.me.first_name} | [ {AYIIN42.me.id} ]"
            )
            client_id.append(AYIIN42.me.id)
        except Exception:
            pass
    if AYIIN43:
        try:
            await AYIIN43.start()
            clients.append(43)
            AYIIN43.me = await AYIIN43.get_me()
            #AYIIN43.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN43 in {AYIIN43.me.first_name} | [ {AYIIN43.me.id} ]"
            )
            client_id.append(AYIIN43.me.id)
        except Exception:
            pass
    if AYIIN44:
        try:
            await AYIIN44.start()
            clients.append(44)
            AYIIN44.me = await AYIIN44.get_me()
            #AYIIN44.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN44 in {AYIIN44.me.first_name} | [ {AYIIN44.me.id} ]"
            )
            client_id.append(AYIIN44.me.id)
        except Exception:
            pass
    if AYIIN45:
        try:
            await AYIIN45.start()
            clients.append(45)
            AYIIN45.me = await AYIIN45.get_me()
            #AYIIN45.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN45 in {AYIIN45.me.first_name} | [ {AYIIN45.me.id} ]"
            )
            client_id.append(AYIIN45.me.id)
        except Exception:
            pass
    if AYIIN46:
        try:
            await AYIIN46.start()
            clients.append(46)
            AYIIN46.me = await AYIIN46.get_me()
            #AYIIN46.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN46 in {AYIIN46.me.first_name} | [ {AYIIN46.me.id} ]"
            )
            client_id.append(AYIIN46.me.id)
        except Exception:
            pass
    if AYIIN47:
        try:
            await AYIIN47.start()
            clients.append(47)
            AYIIN47.me = await AYIIN47.get_me()
            #AYIIN47.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN47 in {AYIIN47.me.first_name} | [ {AYIIN47.me.id} ]"
            )
            client_id.append(AYIIN47.me.id)
        except Exception:
            pass
    if AYIIN48:
        try:
            await AYIIN48.start()
            clients.append(48)
            AYIIN48.me = await AYIIN48.get_me()
            #AYIIN48.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN48 in {AYIIN48.me.first_name} | [ {AYIIN48.me.id} ]"
            )
            client_id.append(AYIIN48.me.id)
        except Exception:
            pass
    if AYIIN49:
        try:
            await AYIIN49.start()
            clients.append(49)
            AYIIN49.me = await AYIIN49.get_me()
            #AYIIN49.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN49 in {AYIIN49.me.first_name} | [ {AYIIN49.me.id} ]"
            )
            client_id.append(AYIIN49.me.id)
        except Exception:
            pass
    if AYIIN50:
        try:
            await AYIIN50.start()
            clients.append(50)
            AYIIN50.me = await AYIIN50.get_me()
            #AYIIN50.has_a_bot = True if tgbot else False
            logs.info(
                f"AYIIN50 in {AYIIN50.me.first_name} | [ {AYIIN50.me.id} ]"
            )
            client_id.append(AYIIN50.me.id)
        except Exception:
            pass
    logs.info(f"Connecting Database To {adB.name}")
    if adB.ping():
        logs.info(f"Succesfully Connect On {adB.name}")
    logs.info(
        f"Connect On [ {HOSTED_ON} ]\n"
    )



# Ultroid - UserBot
# Copyright (C) 2021-2022 TeamUltroid
#
# This file is a part of < https://github.com/TeamUltroid/Ultroid/ >
# PLease read the GNU Affero General Public License in
# <https://github.com/TeamUltroid/pyUltroid/blob/main/LICENSE>.


from .. import adB


def get_flood():
    return adB.get_key("ANTIFLOOD") or {}


def set_flood(chat_id, limit):
    omk = get_flood()
    omk.update({chat_id: limit})
    return adB.set_key("ANTIFLOOD", omk)


def get_flood_limit(chat_id):
    omk = get_flood()
    if chat_id in omk.keys():
        return omk[chat_id]


def rem_flood(chat_id):
    omk = get_flood()
    if chat_id in omk.keys():
        del omk[chat_id]
        return adB.set_key("ANTIFLOOD", omk)



# py - Ayiin
# Copyright (C) 2022-2023 @AyiinXd
#
# This file is a part of < https://github.com/AyiinXd/pyAyiin >
# PLease read the GNU Affero General Public License in
# <https://www.github.com/AyiinXd/pyAyiin/blob/main/LICENSE/>.
#
# FROM py-Ayiin <https://github.com/AyiinXd/pyAyiin>
# t.me/AyiinChat & t.me/AyiinSupport


# ========================×========================
#            Jangan Hapus Credit Ngentod
# ========================×========================

import asyncio
import os


from fipper import Client
from fipper.types import CallbackQuery, Message
from fipper.raw.functions.channels import GetFullChannel
from fipper.raw.functions.messages import GetFullChat
from fipper.raw.functions.phone import CreateGroupCall, EditGroupCallTitle
from fipper.raw.types import InputPeerChannel, InputPeerChat

from ..config import Var as Variable
from ..methods.queue import Queues

from .client import *

try:
    from pytgcalls.exceptions import GroupCallNotFoundError
except ImportError:
    pass
    GroupCallNotFoundError = None


Var = Variable()


class ClientCalls(object):
    async def StartAud(self, video_ids):
        if AYIIN1:
            try:
                await AYIIN1.group_call.start_audio(video_ids)
            except BaseException as ex:
                await AYIIN1.send_message(
                    Var.LOG_CHAT,
                    f'KESALAHAN: {ex}',
                )
        if AYIIN2:
            try:
                await AYIIN2.group_call.start_audio(video_ids)
            except BaseException as ex:
                await AYIIN2.send_message(
                    Var.LOG_CHAT,
                    f'KESALAHAN: {ex}',
                )
        if AYIIN3:
            try:
                await AYIIN3.group_call.start_audio(video_ids)
            except BaseException as ex:
                await AYIIN3.send_message(
                    Var.LOG_CHAT,
                    f'KESALAHAN: {ex}',
                )
        if AYIIN4:
            try:
                await AYIIN4.group_call.start_audio(video_ids)
            except BaseException as ex:
                await AYIIN4.send_message(
                    Var.LOG_CHAT,
                    f'KESALAHAN: {ex}',
                )
        if AYIIN5:
            try:
                await AYIIN5.group_call.start_audio(video_ids)
            except BaseException as ex:
                await AYIIN5.send_message(
                    Var.LOG_CHAT,
                    f'KESALAHAN: {ex}',
                )
        if AYIIN6:
            try:
                await AYIIN6.group_call.start_audio(video_ids)
            except BaseException as ex:
                await AYIIN6.send_message(
                    Var.LOG_CHAT,
                    f'KESALAHAN: {ex}',
                )
        if AYIIN7:
            try:
                await AYIIN7.group_call.start_audio(video_ids)
            except BaseException as ex:
                await AYIIN7.send_message(
                    Var.LOG_CHAT,
                    f'KESALAHAN: {ex}',
                )
        if AYIIN8:
            try:
                await AYIIN8.group_call.start_audio(video_ids)
            except BaseException as ex:
                await AYIIN8.send_message(
                    Var.LOG_CHAT,
                    f'KESALAHAN: {ex}',
                )
        if AYIIN9:
            try:
                await AYIIN9.group_call.start_audio(video_ids)
            except BaseException as ex:
                await AYIIN9.send_message(
                    Var.LOG_CHAT,
                    f'KESALAHAN: {ex}',
                )
        if AYIIN10:
            try:
                await AYIIN10.group_call.start_audio(video_ids)
            except BaseException as ex:
                await AYIIN10.send_message(
                    Var.LOG_CHAT,
                    f'KESALAHAN: {ex}',
                )
    
    async def StartVid(self, video_ids):
        if AYIIN1:
            try:
                await AYIIN1.group_call.start_video(video_ids)
            except BaseException as ex:
                await AYIIN1.send_message(
                    Var.LOG_CHAT,
                    f'KESALAHAN: {ex}',
                )
        if AYIIN2:
            try:
                await AYIIN2.group_call.start_video(video_ids)
            except BaseException as ex:
                await AYIIN2.send_message(
                    Var.LOG_CHAT,
                    f'KESALAHAN: {ex}',
                )
        if AYIIN3:
            try:
                await AYIIN3.group_call.start_video(video_ids)
            except BaseException as ex:
                await AYIIN3.send_message(
                    Var.LOG_CHAT,
                    f'KESALAHAN: {ex}',
                )
        if AYIIN4:
            try:
                await AYIIN4.group_call.start_video(video_ids)
            except BaseException as ex:
                await AYIIN4.send_message(
                    Var.LOG_CHAT,
                    f'KESALAHAN: {ex}',
                )
        if AYIIN5:
            try:
                await AYIIN5.group_call.start_video(video_ids)
            except BaseException as ex:
                await AYIIN5.send_message(
                    Var.LOG_CHAT,
                    f'KESALAHAN: {ex}',
                )
        if AYIIN6:
            try:
                await AYIIN6.group_call.start_video(video_ids)
            except BaseException as ex:
                await AYIIN6.send_message(
                    Var.LOG_CHAT,
                    f'KESALAHAN: {ex}',
                )
        if AYIIN7:
            try:
                await AYIIN7.group_call.start_video(video_ids)
            except BaseException as ex:
                await AYIIN7.send_message(
                    Var.LOG_CHAT,
                    f'KESALAHAN: {ex}',
                )
        if AYIIN8:
            try:
                await AYIIN8.group_call.start_video(video_ids)
            except BaseException as ex:
                await AYIIN8.send_message(
                    Var.LOG_CHAT,
                    f'KESALAHAN: {ex}',
                )
        if AYIIN9:
            try:
                await AYIIN9.group_call.start_video(video_ids)
            except BaseException as ex:
                await AYIIN9.send_message(
                    Var.LOG_CHAT,
                    f'KESALAHAN: {ex}',
                )
        if AYIIN10:
            try:
                await AYIIN10.group_call.start_video(video_ids)
            except BaseException as ex:
                await AYIIN10.send_message(
                    Var.LOG_CHAT,
                    f'KESALAHAN: {ex}',
                )
    
    async def JoinVc(self, ids):
        if AYIIN1:
            try:
                await AYIIN1.group_call.join(ids)
            except BaseException as ex:
                await AYIIN1.send_message(
                    Var.LOG_CHAT,
                    f'KESALAHAN: {ex}',
                )
        if AYIIN2:
            try:
                await AYIIN2.group_call.join(ids)
            except BaseException as ex:
                await AYIIN2.send_message(
                    Var.LOG_CHAT,
                    f'KESALAHAN: {ex}',
                )
        if AYIIN3:
            try:
                await AYIIN3.group_call.join(ids)
            except BaseException as ex:
                await AYIIN3.send_message(
                    Var.LOG_CHAT,
                    f'KESALAHAN: {ex}',
                )
        if AYIIN4:
            try:
                await AYIIN4.group_call.join(ids)
            except BaseException as ex:
                await AYIIN4.send_message(
                    Var.LOG_CHAT,
                    f'KESALAHAN: {ex}',
                )
        if AYIIN5:
            try:
                await AYIIN5.group_call.join(ids)
            except BaseException as ex:
                await AYIIN5.send_message(
                    Var.LOG_CHAT,
                    f'KESALAHAN: {ex}',
                )
        if AYIIN6:
            try:
                await AYIIN6.group_call.join(ids)
            except BaseException as ex:
                await AYIIN6.send_message(
                    Var.LOG_CHAT,
                    f'KESALAHAN: {ex}',
                )
        if AYIIN7:
            try:
                await AYIIN7.group_call.join(ids)
            except BaseException as ex:
                await AYIIN7.send_message(
                    Var.LOG_CHAT,
                    f'KESALAHAN: {ex}',
                )
        if AYIIN8:
            try:
                await AYIIN8.group_call.join(ids)
            except BaseException as ex:
                await AYIIN8.send_message(
                    Var.LOG_CHAT,
                    f'KESALAHAN: {ex}',
                )
        if AYIIN9:
            try:
                await AYIIN9.group_call.join(ids)
            except BaseException as ex:
                await AYIIN9.send_message(
                    Var.LOG_CHAT,
                    f'KESALAHAN: {ex}',
                )
        if AYIIN10:
            try:
                await AYIIN10.group_call.join(ids)
            except BaseException as ex:
                await AYIIN10.send_message(
                    Var.LOG_CHAT,
                    f'KESALAHAN: {ex}',
                )
    
    async def LeaveVc(self, video_ids):
        if AYIIN1:
            try:
                await AYIIN1.group_call.leave()
            except BaseException as ex:
                await AYIIN1.send_message(
                    Var.LOG_CHAT,
                    f'KESALAHAN: {ex}',
                )
        if AYIIN2:
            try:
                await AYIIN2.group_call.leave()
            except BaseException as ex:
                await AYIIN2.send_message(
                    Var.LOG_CHAT,
                    f'KESALAHAN: {ex}',
                )
        if AYIIN3:
            try:
                await AYIIN3.group_call.leave()
            except BaseException as ex:
                await AYIIN3.send_message(
                    Var.LOG_CHAT,
                    f'KESALAHAN: {ex}',
                )
        if AYIIN4:
            try:
                await AYIIN4.group_call.leave()
            except BaseException as ex:
                await AYIIN4.send_message(
                    Var.LOG_CHAT,
                    f'KESALAHAN: {ex}',
                )
        if AYIIN5:
            try:
                await AYIIN5.group_call.leave()
            except BaseException as ex:
                await AYIIN5.send_message(
                    Var.LOG_CHAT,
                    f'KESALAHAN: {ex}',
                )
        if AYIIN6:
            try:
                await AYIIN6.group_call.leave()
            except BaseException as ex:
                await AYIIN6.send_message(
                    Var.LOG_CHAT,
                    f'KESALAHAN: {ex}',
                )
        if AYIIN7:
            try:
                await AYIIN7.group_call.leave()
            except BaseException as ex:
                await AYIIN7.send_message(
                    Var.LOG_CHAT,
                    f'KESALAHAN: {ex}',
                )
        if AYIIN8:
            try:
                await AYIIN8.group_call.leave()
            except BaseException as ex:
                await AYIIN8.send_message(
                    Var.LOG_CHAT,
                    f'KESALAHAN: {ex}',
                )
        if AYIIN9:
            try:
                await AYIIN9.group_call.leave()
            except BaseException as ex:
                await AYIIN9.send_message(
                    Var.LOG_CHAT,
                    f'KESALAHAN: {ex}',
                )
        if AYIIN10:
            try:
                await AYIIN10.group_call.leave()
            except BaseException as ex:
                await AYIIN10.send_message(
                    Var.LOG_CHAT,
                    f'KESALAHAN: {ex}',
                )


class GroupCalls(ClientCalls, Queues):
    def __init__(self):
        self.chat_id = []
        self.clients = {}
        self.active_calls = []
        self.msgid_cache = {}
        self.play_on = {}
    
    async def TitleVc(self, client, m, title: str):
        peer = await client.resolve_peer(m.chat.id)
        if isinstance(peer, InputPeerChannel):
            chat = await client.send(GetFullChannel(channel=peer))
        if isinstance(peer, InputPeerChat):
            chat = await client.send(GetFullChat(chat_id=peer.chat_id))
        return await client.send(
            EditGroupCallTitle(
                call=chat.full_chat.call,
                title=title,
            )
        )

    async def StartVc(self, client, m, title=None):
        peer = await client.resolve_peer(m.chat.id)
        await client.send(
            CreateGroupCall(
                peer=InputPeerChannel(
                    channel_id=peer.channel_id,
                    access_hash=peer.access_hash,
                ),
                random_id=client.rnd_id() // 9000000000,
            )
        )
        titt = title if title else "🎧 Ayiin Music 🎧"
        await self.TitleVc(client, m, title=titt)
    
    async def startCall(self, client, msg):
        if self.play_on:
            for chats in self.play_on:
                await self.play_on[chats].stop()
            self.play_on.clear()
            await asyncio.sleep(3)
        if msg.video:
            for chats in list(self.clients):
                if chats != msg.chat.id:
                    await self.clients[chats].stop()
                    del self.clients[chats]
            self.play_on.update({self._chat: client.group_call})
        if msg.chat.id not in self.active_calls:
            try:
                client.group_call.on_network_status_changed(
                    self.on_network_changed)
                client.group_call.on_playout_ended(self.playout_ended_handler)
                await client.group_call.join(msg.chat.id)
            except GroupCallNotFoundError as er:
                await msg.reply(er)
                dn, err = await self.StartVc(client, msg)
                if err:
                    return False, err
            except Exception as e:
                await msg.reply(e)
                return False, e
        return True, None

    async def on_network_changed(self, msg, call, is_connected):
        chats = msg.chat.id
        if is_connected:
            if chats not in self.active_calls:
                self.active_calls.append(chats)
        elif chats in self.active_calls:
            self.active_calls.remove(chats)

    async def playout_ended_handler(self, client, msg, call, source, mtype):
        if os.path.exists(source):
            os.remove(source)
        await self.play_from_queue(client, msg)

    async def play_from_queue(self, client, msg):
        chat_id = msg.chat.id
        if chat_id in self.play_on:
            await client.group_call.stop_video()
            self.play_on.pop(chat_id)
        try:
            ppk = await self.skip_song(chat_id)
            if ppk == 0:
                await self.LeaveVc()
            elif ppk == 1:
                await msg.reply("**Antrian Udah Kosong Tod, Gua Cabut Dari OS**")
            else:
                await msg.reply(
                        f"**• Memutar** - [{ppk[0]}]({ppk[1]}) | `{ppk[2]}`",
                        disable_web_page_preview=True,
                    )
        except Exception as e:
            await msg.reply(e)

    async def JoinVcg(self, client, msg):
        chat_id = msg.chat.id
        done, err = await self.startCall(client, msg)

        if done:
            return True
        await msg.reply(f"KESALAHAN saat Bergabung ke Obrolan Suara - {chat_id} :\n{err}")
        return False
                


# Ultroid - UserBot
# Copyright (C) 2021-2022 TeamUltroid
#
# This file is a part of < https://github.com/TeamUltroid/Ultroid/ >
# PLease read the GNU Affero General Public License in
# <https://github.com/TeamUltroid/pyUltroid/blob/main/LICENSE>.

from .. import adB


def get_source_channels():  # Returns List
    return adB.get_key("CH_SOURCE") or []


def get_no_source_channels():  # Returns List
    channels = adB.get_key("CH_SOURCE") or []
    return len(channels)


def is_source_channel_added(id_):
    channels = get_source_channels()
    return id_ in channels


def add_source_channel(id_):  # Take int or str with numbers only , Returns Boolean
    channels = get_source_channels()
    if id_ not in channels:
        channels.append(id_)
        adB.set_key("CH_SOURCE", channels)
    return True


def rem_source_channel(id_):
    channels = get_source_channels()
    if id_ in channels:
        channels.remove(id_)
        adB.set_key("CH_SOURCE", channels)
    return True


#########################


def get_destinations():  # Returns List
    return adB.get_key("CH_DESTINATION") or []


def get_no_destinations():  # Returns List
    channels = adB.get_key("CH_DESTINATION") or []
    return len(channels)


def is_destination_added(id_):
    channels = get_destinations()
    return id_ in channels


def add_destination(id_):  # Take int or str with numbers only , Returns Boolean
    channels = get_destinations()
    if id_ not in channels:
        channels.append(id_)
        adB.set_key("CH_DESTINATION", channels)
    return True


def rem_destination(id_):
    channels = get_destinations()
    if id_ in channels:
        channels.remove(id_)
        adB.set_key("CH_DESTINATION", channels)
    return True



# Ultroid - UserBot
# Copyright (C) 2021-2022 TeamUltroid
#
# This file is a part of < https://github.com/TeamUltroid/Ultroid/ >
# PLease read the GNU Affero General Public License in
# <https://github.com/TeamUltroid/pyUltroid/blob/main/LICENSE>.

import asyncio

from .changer import Changers

try:
    from youtubesearchpython import VideosSearch
except ImportError:
    print("'youtube-search-python' tidak terinstall\nmungkin beberapa modul tidak akan berjalan")
    VideosSearch = None


yins = Changers()


class Funci(object):
    def yt_info_query(self, query: str):
        results = VideosSearch(query, limit=1)
        for result in results.result()["result"]:
            title = result["title"]
            duration_min = result["duration"]
            thumbnail = result["thumbnails"][0]["url"].split("?")[0]
            videoid = result["id"]
            if str(duration_min) == "None":
                duration_sec = 0
            else:
                duration_sec = int(yins.time_to_seconds(duration_min))
        return title, duration_min, duration_sec, thumbnail, videoid
    
    def yt_info_id(self, videoid):
        url = f"https://www.youtube.com/watch?v={videoid}"
        results = VideosSearch(url, limit=1)
        for result in results.result()["result"]:
            title = result["title"]
            duration_min = result["duration"]
            thumbnail = result["thumbnails"][0]["url"].split("?")[0]
            if str(duration_min) == "None":
                duration_sec = 0
            else:
                duration_sec = int(yins.time_to_seconds(duration_min))
        return title, duration_min, duration_sec, thumbnail
    
    def yt_info_query_slider(self, query: str, query_type: int):
        a = VideosSearch(query, limit=10)
        result = (a.result()).get("result")
        title = result[query_type]["title"]
        duration_min = result[query_type]["duration"]
        videoid = result[query_type]["id"]
        thumbnail = result[query_type]["thumbnails"][0]["url"].split("?")[0]
        if str(duration_min) == "None":
            duration_sec = 0
        else:
            duration_sec = int(yins.time_to_seconds(duration_min))
        return title, duration_min, duration_sec, thumbnail, videoid


    async def get_m3u8(self, videoid):
        link = f"https://www.youtube.com/watch?v={videoid}"
        proc = await asyncio.create_subprocess_exec(
            "yt-dlp",
            "-g",
            "-f",
            "best[height<=?720][width<=?1280]",
            f"{link}",
            stdout=asyncio.subprocess.PIPE,
            stderr=asyncio.subprocess.PIPE,
        )
        stdout, stderr = await proc.communicate()
        if stdout:
            return 1, stdout.decode().split("\n")[0]
        else:
            return 0, stderr.decode()



# py - Ayiin
# Copyright (C) 2022-2023 @AyiinXd
#
# This file is a part of < https://github.com/AyiinXd/pyAyiin >
# PLease read the GNU Affero General Public License in
# <https://www.github.com/AyiinXd/pyAyiin/blob/main/LICENSE/>.
#
# FROM py-Ayiin <https://github.com/AyiinXd/pyAyiin>
# t.me/AyiinChat & t.me/AyiinSupport


# ========================×========================
#            Jangan Hapus Credit Ngentod
# ========================×========================

class Changers(object):
    async def int_to_alpha(self, user_id: int) -> str:
        alphabet = ["a", "b", "c", "d", "e", "f", "g", "h", "i", "j"]
        text = ""
        user_id = str(user_id)
        for i in user_id:
            text += alphabet[int(i)]
        return text


    async def alpha_to_int(self, user_id_alphabet: str) -> int:
        alphabet = ["a", "b", "c", "d", "e", "f", "g", "h", "i", "j"]
        user_id = ""
        for i in user_id_alphabet:
            index = alphabet.index(i)
            user_id += str(index)
        user_id = int(user_id)
        return user_id


    def time_to_seconds(self, time):
        stringt = str(time)
        return sum(
            int(x) * 60 ** i for i, x in enumerate(reversed(stringt.split(":")))
        )


    def seconds_to_min(self, seconds):
        if seconds is not None:
            seconds = int(seconds)
            d, h, m, s = (
                seconds // (3600 * 24),
                seconds // 3600 % 24,
                seconds % 3600 // 60,
                seconds % 3600 % 60,
            )
            if d > 0:
                return "{:02d}:{:02d}:{:02d}:{:02d}".format(d, h, m, s)
            elif h > 0:
                return "{:02d}:{:02d}:{:02d}".format(h, m, s)
            elif m > 0:
                return "{:02d}:{:02d}".format(m, s)
            elif s > 0:
                return "00:{:02d}".format(s)
        return "-"
